﻿'Copyright (c) 2008, William Severance, Jr., WESNet Designs
'All rights reserved.

'Redistribution and use in source and binary forms, with or without modification, are permitted
'provided that the following conditions are met:

'Redistributions of source code must retain the above copyright notice, this list of conditions
'and the following disclaimer.

'Redistributions in binary form must reproduce the above copyright notice, this list of conditions
'and the following disclaimer in the documentation and/or other materials provided with the distribution.

'Neither the name of William Severance, Jr. or WESNet Designs may be used to endorse or promote
'products derived from this software without specific prior written permission.

'Disclaimer: THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
'            OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
'            AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER BE LIABLE
'            FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
'            LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
'            INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
'            OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
'            IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'Although I will try to answer questions regarding the installation and use of this software when
'such questions are submitted via e-mail to the below address, no promise of further
'support or enhancement is made nor should be assumed.

'Developer Contact Information:
'     William Severance, Jr.
'     WESNet Designs
'     679 Upper Ridge Road
'     Bridgton, ME 04009
'     Phone: 207-647-9375
'     E-Mail: bill@wesnetdesigns.com
'     Website: www.wesnetdesigns.com


Namespace WESNet.AJAX.Controls.Attributes

    Public MustInherit Class ClientAttributeBase
        Inherits System.Attribute
        Private _IsApplied As Boolean
        Private _ClientAlias As String

        Public Sub New()
            _IsApplied = True
            _ClientAlias = String.Empty
        End Sub

        Public Property ClientAlias() As String
            Get
                Return _ClientAlias
            End Get
            Set(ByVal value As String)
                _ClientAlias = value
            End Set
        End Property

        Public Property IsApplied() As Boolean
            Get
                Return _IsApplied
            End Get
            Set(ByVal value As Boolean)
                _IsApplied = value
            End Set
        End Property

        Public Overrides Function Equals(ByVal obj As Object) As Boolean
            If Object.ReferenceEquals(obj, Me) Then Return True

            If (TypeOf obj Is ClientAttributeBase) Then
                Dim other As ClientAttributeBase = DirectCast(obj, ClientAttributeBase)
                Return other._IsApplied = _IsApplied AndAlso other._ClientAlias = _ClientAlias
            End If
            Return False

        End Function

        Public Overrides Function GetHashCode() As Integer
            Return _IsApplied.GetHashCode() + _ClientAlias.GetHashCode()
        End Function

        Public Overrides Function IsDefaultAttribute() As Boolean
            Return Not _IsApplied
        End Function

    End Class

    <AttributeUsage(AttributeTargets.Property)> _
    Public NotInheritable Class ClientPropertyAttribute
        Inherits ClientAttributeBase

        Public Sub New(ByVal IsApplied As Boolean, ByVal ClientAlias As String)
            MyBase.IsApplied = IsApplied
            MyBase.ClientAlias = ClientAlias
        End Sub

        Public Sub New(ByVal ClientAlias As String)
            MyBase.IsApplied = True
            MyBase.ClientAlias = ClientAlias
        End Sub

        Public Sub New(ByVal IsApplied As Boolean)
            MyBase.IsApplied = IsApplied
            MyBase.ClientAlias = String.Empty
        End Sub

        Public Sub New()
            MyBase.IsApplied = True
            MyBase.ClientAlias = String.Empty
        End Sub
    End Class

    <AttributeUsage(AttributeTargets.Property)> _
    Public NotInheritable Class ClientEventAttribute
        Inherits ClientAttributeBase

        Public Sub New(ByVal IsApplied As Boolean, ByVal ClientAlias As String)
            MyBase.IsApplied = IsApplied
            MyBase.ClientAlias = ClientAlias
        End Sub

        Public Sub New(ByVal ClientAlias As String)
            MyBase.IsApplied = True
            MyBase.ClientAlias = ClientAlias
        End Sub

        Public Sub New(ByVal IsApplied As Boolean)
            MyBase.IsApplied = IsApplied
            MyBase.ClientAlias = String.Empty
        End Sub

        Public Sub New()
            MyBase.IsApplied = True
            MyBase.ClientAlias = String.Empty
        End Sub
    End Class

    <AttributeUsage(AttributeTargets.Property)> _
    Public NotInheritable Class ClientElementAttribute
        Inherits ClientAttributeBase

        Public Sub New(ByVal IsApplied As Boolean, ByVal ClientAlias As String)
            MyBase.IsApplied = IsApplied
            MyBase.ClientAlias = ClientAlias
        End Sub

        Public Sub New(ByVal ClientAlias As String)
            MyBase.IsApplied = True
            MyBase.ClientAlias = ClientAlias
        End Sub

        Public Sub New(ByVal IsApplied As Boolean)
            MyBase.IsApplied = IsApplied
            MyBase.ClientAlias = String.Empty
        End Sub

        Public Sub New()
            MyBase.IsApplied = True
            MyBase.ClientAlias = String.Empty
        End Sub
    End Class

    <AttributeUsage(AttributeTargets.Property)> _
    Public NotInheritable Class ClientIDReferenceAttribute
        Inherits ClientAttributeBase

        Public Sub New(ByVal IsApplied As Boolean, ByVal ClientAlias As String)
            MyBase.IsApplied = IsApplied
            MyBase.ClientAlias = ClientAlias
        End Sub

        Public Sub New(ByVal ClientAlias As String)
            MyBase.IsApplied = True
            MyBase.ClientAlias = ClientAlias
        End Sub

        Public Sub New(ByVal IsApplied As Boolean)
            MyBase.IsApplied = IsApplied
            MyBase.ClientAlias = String.Empty
        End Sub

        Public Sub New()
            MyBase.IsApplied = True
            MyBase.ClientAlias = String.Empty
        End Sub
    End Class

    <AttributeUsage(AttributeTargets.Property)> _
    Public NotInheritable Class ClientUrlAttribute
        Inherits ClientAttributeBase

        Public Sub New(ByVal IsApplied As Boolean, ByVal ClientAlias As String)
            MyBase.IsApplied = IsApplied
            MyBase.ClientAlias = ClientAlias
        End Sub

        Public Sub New(ByVal ClientAlias As String)
            MyBase.IsApplied = True
            MyBase.ClientAlias = ClientAlias
        End Sub

        Public Sub New(ByVal IsApplied As Boolean)
            MyBase.IsApplied = IsApplied
            MyBase.ClientAlias = String.Empty
        End Sub

        Public Sub New()
            MyBase.IsApplied = True
            MyBase.ClientAlias = String.Empty
        End Sub
    End Class

End Namespace