'Copyright (c) 2008, William Severance, Jr., WESNet Designs
'All rights reserved.

'Redistribution and use in source and binary forms, with or without modification, are permitted
'provided that the following conditions are met:

'Redistributions of source code must retain the above copyright notice, this list of conditions
'and the following disclaimer.

'Redistributions in binary form must reproduce the above copyright notice, this list of conditions
'and the following disclaimer in the documentation and/or other materials provided with the distribution.

'Neither the name of William Severance, Jr. or WESNet Designs may be used to endorse or promote
'products derived from this software without specific prior written permission.

'Disclaimer: THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
'            OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
'            AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER BE LIABLE
'            FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
'            LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
'            INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
'            OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
'            IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'Although I will try to answer questions regarding the installation and use of this software when
'such questions are submitted via e-mail to the below address, no promise of further
'support or enhancement is made nor should be assumed.

'Developer Contact Information:
'     William Severance, Jr.
'     WESNet Designs
'     679 Upper Ridge Road
'     Bridgton, ME 04009
'     Phone: 207-647-9375
'     E-Mail: bill@wesnetdesigns.com
'     Website: www.wesnetdesigns.com


Imports System.Web.UI

Namespace WESNet.AJAX.Controls.ImageEditor

    Public Class ToolBarButton
        Inherits System.Web.UI.WebControls.ImageButton

#Region "Constants"
        Public Const ButtonWidth As Integer = 25
        Public Const ButtonHeight As Integer = 25
        Public Const ButtonPadding As Integer = 2
        Public Const ButtonFrameWidth As Integer = ButtonWidth + 2 * ButtonPadding
        Public Const ButtonFrameHeight As Integer = ButtonHeight + 2 * ButtonPadding
#End Region

#Region "Private Members"
        Private _OnMouseOver As String
        Private _OnMouseOut As String
#End Region

#Region "Public Properties"

        Public ReadOnly Property OnMouseOver() As String
            Get
                Return _OnMouseOver
            End Get
        End Property

        Public ReadOnly Property OnMouseOut() As String
            Get
                Return _OnMouseOut
            End Get
        End Property
#End Region

#Region "Constructors"

        Public Sub New(ByVal ie As WESNet.AJAX.Controls.ImageEditor.ImageEditor, ByVal CommandName As String, ByVal Tooltip As String)
            Me.New(ie, CommandName, Tooltip, Nothing, Nothing)
        End Sub

        Public Sub New(ByVal ie As WESNet.AJAX.Controls.ImageEditor.ImageEditor, ByVal CommandName As String, ByVal Tooltip As String, ByVal OnClientClick As String)
            MyBase.ID = "btn" & CommandName
            MyBase.CommandName = CommandName
            MyBase.OnClientClick = OnClientClick
            MyBase.ToolTip = Tooltip
            MyBase.ImageUrl = ie.Page.ClientScript.GetWebResourceUrl(ie.GetType, CommandName & ".jpg")
            MyBase.Style.Add("border", "solid white 1px")
            MyBase.Width = System.Web.UI.WebControls.Unit.Pixel(ButtonWidth)
            MyBase.Height = System.Web.UI.WebControls.Unit.Pixel(ButtonHeight)
            _OnMouseOver = ie.GetScriptMethodReference("mouseOver(this)")
            _OnMouseOut = ie.GetScriptMethodReference("mouseOut(this)")
        End Sub

        Public Sub New(ByVal ie As WESNet.AJAX.Controls.ImageEditor.ImageEditor, ByVal CommandName As String, ByVal Tooltip As String, ByVal ImageSrc As String, ByVal OnClientClick As String)
            Me.New(ie, CommandName, Tooltip, OnClientClick)
            If ImageSrc Is Nothing Then
                MyBase.ImageUrl = ie.Page.ClientScript.GetWebResourceUrl(ie.GetType, CommandName & ".jpg")
            Else
                MyBase.ImageUrl = ImageSrc
            End If
        End Sub

        Public Sub New(ByVal ie As WESNet.AJAX.Controls.ImageEditor.ImageEditor, ByVal CommandName As String)
            Me.New(ie, CommandName, Nothing, Nothing)
        End Sub
#End Region

#Region "Methods"

        Protected Overrides Sub AddAttributesToRender(ByVal writer As System.Web.UI.HtmlTextWriter)
            MyBase.AddAttributesToRender(writer)
            If Not String.IsNullOrEmpty(OnMouseOver) Then writer.AddAttribute("onmouseover", OnMouseOver)
            If Not String.IsNullOrEmpty(OnMouseOut) Then writer.AddAttribute("onmouseout", OnMouseOut)
            MyBase.Style.Add("margin", "0px auto")
        End Sub

#End Region

    End Class

End Namespace
