﻿'Copyright (c) 2008, William Severance, Jr., WESNet Designs
'All rights reserved.

'Redistribution and use in source and binary forms, with or without modification, are permitted
'provided that the following conditions are met:

'Redistributions of source code must retain the above copyright notice, this list of conditions
'and the following disclaimer.

'Redistributions in binary form must reproduce the above copyright notice, this list of conditions
'and the following disclaimer in the documentation and/or other materials provided with the distribution.

'Neither the name of William Severance, Jr. or WESNet Designs may be used to endorse or promote
'products derived from this software without specific prior written permission.

'Disclaimer: THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
'            OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
'            AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER BE LIABLE
'            FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
'            LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
'            INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
'            OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
'            IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

'Although I will try to answer questions regarding the installation and use of this software when
'such questions are submitted via e-mail to the below address, no promise of further
'support or enhancement is made nor should be assumed.

'Developer Contact Information:
'     William Severance, Jr.
'     WESNet Designs
'     679 Upper Ridge Road
'     Bridgton, ME 04009
'     Phone: 207-647-9375
'     E-Mail: bill@wesnetdesigns.com
'     Website: www.wesnetdesigns.com

Imports DotNetNuke
Imports DotNetNuke.Services.Exceptions
Imports DotNetNuke.Services.Scheduling
Imports DotNetNuke.Common.Utilities
Imports System.IO

Namespace WESNet.DNN.Controls.ImageEditorControl
    Public Class PurgeImageEditorCacheFiles
        Inherits SchedulerClient

        Public Sub New(ByVal objScheduleHistoryItem As DotNetNuke.Services.Scheduling.ScheduleHistoryItem)
            MyBase.new()
            Me.ScheduleHistoryItem = objScheduleHistoryItem
        End Sub

        Public Overrides Sub DoWork()
            Dim cacheFilepath As String
            Dim dirInfo As DirectoryInfo
            Dim expiredAfter As DateTime = Now.AddMinutes(-CDbl(Defaults.CacheFileLifetimeMinutes))
            Dim filesPurged As Integer = 0
            Dim filesProcessed As Integer = 0
            Dim pc As Entities.Portals.PortalController
            Try
                Me.Progressing()
                pc = New Entities.Portals.PortalController
                For Each Portal As Entities.Portals.PortalInfo In pc.GetPortals()
                    cacheFilepath = Portal.HomeDirectoryMapPath() & Defaults.ImageEditorCacheFolder
                    If Directory.Exists(cacheFilepath) Then
                        dirInfo = New DirectoryInfo(cacheFilepath)
                        For Each f As FileInfo In dirInfo.GetFiles("*.resource", SearchOption.TopDirectoryOnly)
                            filesProcessed += 1
                            If f.LastAccessTime < expiredAfter Then
                                Utilities.DeleteFile(f.FullName, Portal.PortalID, False)
                                filesPurged += 1
                            End If
                        Next
                    End If
                    FileSystemUtils.SynchronizeFolder(Portal.PortalID, cacheFilepath, Defaults.ImageEditorCacheFolder, False)
                Next
                'update the result to success since no exception was thrown
                Me.ScheduleHistoryItem.Succeeded = True
                Me.ScheduleHistoryItem.AddLogNote(String.Format("{0} of {1} ImageEditorCache files purged.", FilesPurged, FilesProcessed))
            Catch exc As Exception
                Me.ScheduleHistoryItem.Succeeded = False
                Me.ScheduleHistoryItem.AddLogNote("ImageEditorCache file purge failed. " + exc.ToString)
                Me.Errored(exc)  'notification that we have errored
                LogException(exc) 'log the exception
            End Try
        End Sub
    End Class
End Namespace
